/* (c) 2024 Vladimir Estivill-Castro 	*/
/* Apache-2.0 License			*/
#include <functional>
#include <memory>

#include "rclcpp/rclcpp.hpp"
#include "turtlesim/msg/pose.hpp" 
#include "turtle_sensor_interface/srv/turtle_position.hpp"

using std::placeholders::_1;

/** holder of the x position, updated in the subscriber call back, returned in
the server
 */
double x_Position;
/** holder of the y position, updated in the subscriber call back, returned in
the server
 */
double y_Position;
/** holder of the orientation, updated in the subscriber call back, returned in
 the server
 */
double a_theta;

std::shared_ptr<rclcpp::Node> a_node;

void position_service(const std::shared_ptr<turtle_sensor_interface::srv::TurtlePosition::Request> request,     
          std::shared_ptr<turtle_sensor_interface::srv::TurtlePosition::Response>       response)  
  { response->x = x_Position; 
    response->y = y_Position;
    response->theta = a_theta;
  }

class PoseListener : public rclcpp::Node
{
public:
  PoseListener() : Node("turtle_pose_service")
  {
    subscription_ = this->create_subscription<turtlesim::msg::Pose>(
      "/turtle1/pose", 10, std::bind(&PoseListener::topic_callback, this, _1));
    service_ = this->create_service<turtle_sensor_interface::srv::TurtlePosition>("turtle_pose_service",  &position_service);
  }

private:
  void topic_callback(const turtlesim::msg::Pose& msg) const
  { //RCLCPP_INFO(this->get_logger(), "The turtle's pose is: (x=%2.1f, y=%2.1f, theta=%2.1f", msg.x, msg.  y, msg.theta); 
    x_Position = msg.x;
    y_Position = msg.y;
    a_theta = msg.theta;
  }
  rclcpp::Subscription<turtlesim::msg::Pose>::SharedPtr subscription_;
  rclcpp::Service<turtle_sensor_interface::srv::TurtlePosition>::SharedPtr service_; 
};


int main(int argc, char * argv[])
{
  rclcpp::init(argc, argv);

  rclcpp::spin(std::make_shared<PoseListener>());

  rclcpp::shutdown();
  return 0;
}
